/**
 * @file
 * File quiz_h5p_integration_question.js manages H5P questions display.
 *
 * @category Quiz Question
 *
 * @package Quiz H5P Integration
 *
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */

(function ($, Drupal, window, document) {

  'use strict';

  Drupal.behaviors.quizH5PIntegrationQuestion = {
    attach: function (context, settings) {

      /**
      * Loads user answers into H5P fields if any found.
      */

      var loadUserAnswers = function () {
        if ($.isArray(H5P.instances)) {
          $.each(H5P.instances, function (instanceIndex, instance) {
            if (instance.contentId) {
              var H5PContentId = instance.contentId;
              if (instance.params && instance.params.userAnswers) {
                if ($.isArray(instance.params.userAnswers)) {
                  // We load the user answers.
                  $.each(instance.params.userAnswers, function (answerIndex, answer) {
                    var $quizH5PAnswerField = $('#quiz-h5p-integration-' + H5PContentId + '-answer-' + answerIndex);
                    if ($quizH5PAnswerField.length) {
                      instance.params.userAnswers[answerIndex] = $quizH5PAnswerField.val();
                      if (instance.clozes && instance.clozes[answerIndex]) {
                        instance.clozes[answerIndex].setUserInput(instance.params.userAnswers[answerIndex]);
                      }
                    }
                  });
                }
              }
            }
          });
        }
      };

      /**
      * Updates quiz user answer hidden fields with H5P user answers.
      */
      var updateQuizH5PAnswerFields = function () {
        if ($.isArray(H5P.instances)) {
          $.each(H5P.instances, function (instanceIndex, instance) {
            if (instance.contentId) {
              var H5PContentId = instance.contentId;
              if (instance.params && instance.params.userAnswers) {
                if ($.isArray(instance.params.userAnswers)) {
                  $.each(instance.params.userAnswers, function (answerIndex, answer) {
                    var $quizH5PAnswerField = $('#quiz-h5p-integration-' + H5PContentId + '-answer-' + answerIndex);
                    if ($quizH5PAnswerField.length) {
                      $quizH5PAnswerField.val(answer);
                    }
                  });
                }
              }
            }
          });
        }
      };

      /**
      * Updates quiz user score hidden field with H5P user score.
      *
      * @param {number} currentContentId (optional) the H5P content which score has to be extracted and applied to the hidden field.
      * @param {number} score (optional) the score to set as field value.
      */
      var updateQuizH5PScoreField = function (currentContentId, score) {
        if (!$.isNumeric(score)) {
          if ($.isArray(H5P.instances)) {
            $.each(H5P.instances, function (instanceIndex, instance) {
              if (instance.contentId) {
                var H5PContentId = instance.contentId;
                if ($.isNumeric(currentContentId)) {
                  if (H5PContentId === currentContentId) {
                    if (typeof instance.getScore === 'function') {
                      score = instance.getScore();
                    }
                    return false;
                  }
                }
                else {
                  if (typeof instance.getScore === 'function') {
                    score = instance.getScore();
                    if ($.isNumeric(score)) {
                      var $quizH5PScoreField = $('#quiz-h5p-integration-' + H5PContentId + '-score');
                      if ($quizH5PScoreField.length) {
                        $quizH5PScoreField.val(score);
                      }
                    }
                  }
                }
              }
            });
          }
        }

        if ($.isNumeric(currentContentId) && $.isNumeric(score)) {
          var $quizH5PScoreField = $('#quiz-h5p-integration-' + currentContentId + '-score');
          if ($quizH5PScoreField.length) {
            $quizH5PScoreField.val(score);
          }
        }
      };

      /**
      * Main function. Triggered when DOM is ready.
      */
      var init = function () {
        if (H5P) {
          if (H5P.externalDispatcher) {
            // Each time an event is triggered on an H5P instance.
            H5P.externalDispatcher.on('xAPI', function (event) {
              // Updates quiz user answer hidden fields.
              updateQuizH5PAnswerFields();

              // Gets the current score.
              var score = event.getScore();
              // Gets H5P content Id which triggered the event.
              var currentContentId = event.getVerifiedStatementValue(['object', 'definition', 'extensions', 'http://h5p.org/x-api/h5p-local-content-id']);

              // Updates quiz user score hidden field.
              updateQuizH5PScoreField(currentContentId, score);
            });
          }

          // Updates quiz user answer hidden fields one last time before submitting the form.
          var $quizQuestionAnsweringForm = $('#quiz-question-answering-form');
          if ($quizQuestionAnsweringForm.length) {
            $quizQuestionAnsweringForm.submit(function () {
              updateQuizH5PAnswerFields();
              updateQuizH5PScoreField();
            });
          }

        }
      };

      // Triggering init() function when DOM is ready.
      $(document).ready(function () {
        init();
      });

      // Triggering loadUserAnswers() function when document is loaded.
      $(window).load(function () {
        // Loads old answers if this quiz was already taken.
        loadUserAnswers();
      });
    }
  };

  // Run drupal's attachBehaviors() just to be sure that quizH5PIntegrationQuestion behavior is loaded.
  Drupal.attachBehaviors(document, Drupal.settings);

})(jQuery, Drupal, this, this.document);
